//## begin module%1.4%.codegen_version preserve=yes
//   Read the documentation to learn more about C++ code generator
//   versioning.
//## end module%1.4%.codegen_version

//## begin module%3DE5601503CD.cm preserve=no
//	  %X% %Q% %Z% %W%
//## end module%3DE5601503CD.cm

//## begin module%3DE5601503CD.cp preserve=no
//## end module%3DE5601503CD.cp

//## Module: PlotEfficiency%3DE5601503CD; Package body
//## Subsystem: XSPlot::Commands%3DA1AFD101BF
//## Source file: /local/data/xerxes1/gordon/headas/Xspec/src/XSPlot/Commands/PlotEfficiency.cxx

//## begin module%3DE5601503CD.additionalIncludes preserve=no
//## end module%3DE5601503CD.additionalIncludes

//## begin module%3DE5601503CD.includes preserve=yes
#include <XSPlot/Plot/PlotGroupCreatorClasses.h>
#include <XSPlot/Plot/PlotGroup.h>
#include <XSPlot/Plot/PlotSettings.h>
#include <XSPlot/Plot/PlotStyle.h>
#include <XSPlot/Plot/PlotTypes.h>

#include <XSContainer.h>
#include <XSModel/Data/SpectralData.h>
#include <XSModel/Data/Detector/Response.h>
#include <XSModel/GlobalContainer/DataContainer.h>
#include <XSUtil/Error/Error.h>

#include <algorithm>
#include <cfloat>
//## end module%3DE5601503CD.includes

// PlotEfficiency
#include <XSPlot/Commands/PlotEfficiency.h>
//## begin module%3DE5601503CD.declarations preserve=no
//## end module%3DE5601503CD.declarations

//## begin module%3DE5601503CD.additionalDeclarations preserve=yes
//## end module%3DE5601503CD.additionalDeclarations


// Class PlotEfficiency 

PlotEfficiency::PlotEfficiency()
  //## begin PlotEfficiency::PlotEfficiency%3DE55E3D0007_const.hasinit preserve=no
  //## end PlotEfficiency::PlotEfficiency%3DE55E3D0007_const.hasinit
  //## begin PlotEfficiency::PlotEfficiency%3DE55E3D0007_const.initialization preserve=yes
  : PlotCommand("efficiency", new CreateEfficiencyPlotGroups())
  //## end PlotEfficiency::PlotEfficiency%3DE55E3D0007_const.initialization
{
  //## begin PlotEfficiency::PlotEfficiency%3DE55E3D0007_const.body preserve=yes
   isDataRequired(true);
   isActiveModelRequired(false);
   isDivisibleByArea(false);
   addCompLevel(0);
   isBackgroundApplicable(false);
   
   PlotAttributes styles;
   styles.symbolStyle = PlotStyle::BLANK;
   styles.lineStyle = PlotStyle::DASHDOT;
   setStyleMap(PlotStyle::DATA, styles);
  //## end PlotEfficiency::PlotEfficiency%3DE55E3D0007_const.body
}


PlotEfficiency::~PlotEfficiency()
{
  //## begin PlotEfficiency::~PlotEfficiency%3DE55E3D0007_dest.body preserve=yes
  //## end PlotEfficiency::~PlotEfficiency%3DE55E3D0007_dest.body
}



//## Other Operations (implementation)
void PlotEfficiency::manipulate (std::vector<PlotGroup*>& plotGroups, const PlotSettings& settings)
{
  //## begin PlotEfficiency::manipulate%4B43897E0349.body preserve=yes
   size_t iPlotGroup = 0;
   const size_t nSpec = XSContainer::datasets->numberOfSpectra();
   for (size_t iSpec=1; iSpec<=nSpec; ++iSpec)
   {
     const SpectralData* spectrum = XSContainer::datasets->lookup(iSpec);
     const std::vector<Response*>& detectors = spectrum->detector();
     const size_t nDets = detectors.size();
     for (size_t iDet=0; iDet<nDets; ++iDet)
     {
        const Response* resp = detectors[iDet];
        if (resp)
        {
           PlotGroup* gr = plotGroups[iPlotGroup];
           const RealArray& efficiency = resp->efficiency();
           if (gr->yData.data.size() != efficiency.size())
           {
              throw RedAlert("PlotGroup/Efficiency array size mismatch.");
           }
           const size_t nPts = gr->n;
           for (size_t k=0; k<nPts; ++k)
              gr->yData.data[k] = efficiency[k];
           gr->xAxis.errors.resize(0);
           gr->yData.errors.resize(0);
           ++iPlotGroup;
        }
     }
   }
  //## end PlotEfficiency::manipulate%4B43897E0349.body
}

void PlotEfficiency::setRanges (std::vector<PlotGroup*>& plotGroups, const PlotSettings& settings)
{
  //## begin PlotEfficiency::setRanges%4B43898A00DF.body preserve=yes
   Real xlow=0.0, xhigh=0.0;
   PlotGroupCreator::determineXRange(plotGroups,&xlow,&xhigh);
   
   PlotRange& rangeVals = rangeSettings();
   rangeVals.ranges.x1 = xlow;
   rangeVals.ranges.x2 = xhigh;
   
   Real yMin = DBL_MAX;
   Real yMax = -DBL_MAX;
   for (size_t i=0; i<plotGroups.size(); ++i)
   {
      const PlotGroup* pg = plotGroups[i];
      const size_t nPts = pg->n;
      for (size_t k=0; k<nPts; ++k)
      {
         Real y = pg->yData.data[k];
         yMin = (y < yMin) ? y : yMin;
         yMax = (y > yMax) ? y : yMax;
      }
   }
   // Not sure if this can ever happen:
   if (yMax < yMin)
   {
      throw YellowAlert("Unable to determine Y-axis range for plot efficiency.\n");
   }
   
   rangeVals.xScaleType = settings.xLog() ? PlotStyle::LOG :
                PlotStyle::LINEAR;
   if (settings.yLog())
   {
      rangeVals.yScaleType = PlotStyle::LOG;
      rangeVals.ranges.y1 = std::max(yMin, yMax*1.e-7);
      rangeVals.ranges.y2 = yMax*2.0;
   }
   else
   {
      rangeVals.yScaleType = PlotStyle::LINEAR;
      rangeVals.ranges.y1 = -.01*yMax;
      rangeVals.ranges.y2 = 1.05*yMax;
   }
   
   rangeVals.useRangeFlags = PlotStyle::XMIN + PlotStyle::XMAX +
                              PlotStyle::YMIN + PlotStyle::YMAX;

  //## end PlotEfficiency::setRanges%4B43898A00DF.body
}

void PlotEfficiency::makeLabels (const PlotSettings& settings, StandardLabels& labels) const
{
  //## begin PlotEfficiency::makeLabels%4B43899F0133.body preserve=yes
   labels.title = settings.lookupLabelName("t:efficiency");
   settings.makeXOptionDependentLabel(labels.x);
   labels.y = settings.lookupLabelName("y:efficiency");
  //## end PlotEfficiency::makeLabels%4B43899F0133.body
}

bool PlotEfficiency::requestNewXoption (const PlotSettings& settings, XaxisMode& newOption) const
{
  //## begin PlotEfficiency::requestNewXoption%4B4389A603DF.body preserve=yes
   bool doChange = false;
   if (settings.xOption() == CHANNELS)
   {
      newOption = ENERGY;
      doChange = true;
   }
   return doChange;
  //## end PlotEfficiency::requestNewXoption%4B4389A603DF.body
}

// Additional Declarations
  //## begin PlotEfficiency%3DE55E3D0007.declarations preserve=yes
  //## end PlotEfficiency%3DE55E3D0007.declarations

//## begin module%3DE5601503CD.epilog preserve=yes
//## end module%3DE5601503CD.epilog
